// Generated by CoffeeScript 1.10.0
(function() {
  var _, bodyParser, createControlServer, express, moment, morgan, os;

  express = require('express');

  morgan = require('morgan');

  moment = require('moment');

  _ = require('lodash');

  bodyParser = require('body-parser');

  os = require('os');

  createControlServer = function(app) {
    var api, renderVars, server;
    server = express();
    server.set('view engine', 'pug');
    server.set('views', app.installDir + '/control-views');
    server.use(morgan('[CONTROL] :method :url :status :response-time ms - :res[content-length]'));
    server.use('/components', express["static"](app.installDir + '/bower_components'));
    server.use('/assets', express["static"](app.installDir + '/control-assets'));
    server.get('/', function(req, res) {
      return res.redirect('/devices');
    });
    renderVars = {
      app: app,
      moment: moment,
      _: _
    };
    server.get('/devices', function(req, res) {
      return res.render('device-list', _.defaults({
        devices: app.deviceManager.all(),
        networkInterfaces: os.networkInterfaces()
      }, renderVars));
    });
    server.param('deviceId', function(req, res, next, id) {
      req.deviceId = id;
      if (req.device = app.deviceManager.byDeviceId(id)) {
        return next();
      } else {
        return res.status(404).end();
      }
    });
    server.get('/devices/details/:deviceId', function(req, res) {
      var data;
      data = {
        device: req.device,
        deviceId: req.deviceId
      };
      return res.render('device-details', _.defaults(data, renderVars));
    });
    server.get('/devices/details/:deviceId/start-simulation/:simulationId', function(req, res) {
      req.device.plannedSimulationId = req.simulationId;
      return res.redirect('/devices/details/' + encodeURIComponent(req.deviceId));
    });
    server.get('/simulations', function(req, res) {
      return res.render('simulation-list', _.defaults({
        simulations: app.simulationManager.all()
      }, renderVars));
    });
    server.param('simulationId', function(req, res, next, id) {
      req.simulationId = id;
      if (req.simulation = app.simulationManager.get(id)) {
        return next();
      } else {
        return res.status(404).end();
      }
    });
    server.get('/simulations/edit/:simulationId', function(req, res) {
      var data;
      data = {
        simulation: req.simulation,
        simulationId: req.simulationId
      };
      return res.render('edit-simulation', _.defaults(data, renderVars));
    });
    server.post('/simulations/edit/:simulationId/set-title', bodyParser.urlencoded({
      extended: true
    }), function(req, res) {
      req.simulation.title = req.body.title || req.simulation.title;
      return app.saveConfig(function(err) {
        if (err != null) {
          return res.status(500).end('Failed to save configuration: ' + err);
        } else {
          return res.redirect('/simulations/edit/' + encodeURIComponent(req.simulationId));
        }
      });
    });
    server.param('newTaskType', function(req, res, next, id) {
      req.newTaskType = id;
      return next();
    });
    server.get('/simulations/edit/:simulationId/add-task/:newTaskType', function(req, res) {
      req.simulation.tasks.push({
        type: req.newTaskType
      });
      return app.saveConfig(function(err) {
        if (err != null) {
          return res.status(500).end('Failed to save configuration: ' + err);
        } else {
          return res.redirect('/simulations/edit/' + encodeURIComponent(req.simulationId));
        }
      });
    });
    server.param('taskId', function(req, res, next, id) {
      req.taskId = parseInt(id);
      return next();
    });
    server.post('/simulations/edit/:simulationId/update-task/:taskId', bodyParser.urlencoded({
      extended: true
    }), function(req, res) {
      var task;
      task = req.simulation.tasks[req.taskId];
      if (task && task.type === req.body.type) {
        _.assign(task, req.body);
      }
      return app.saveConfig(function(err) {
        if (err != null) {
          return res.status(500).end('Failed to save configuration: ' + err);
        } else {
          return res.redirect('/simulations/edit/' + encodeURIComponent(req.simulationId));
        }
      });
    });
    server.get('/simulations/edit/:simulationId/delete-task/:taskId', function(req, res) {
      if (req.simulation.tasks[req.taskId] != null) {
        req.simulation.tasks.splice(req.taskId, 1);
      }
      return app.saveConfig(function(err) {
        if (err != null) {
          return res.status(500).end('Failed to save configuration: ' + err);
        } else {
          return res.redirect('/simulations/edit/' + encodeURIComponent(req.simulationId));
        }
      });
    });
    server.get('/simulations/create', function(req, res) {
      var id;
      id = app.simulationManager.create();
      return app.saveConfig(function(err) {
        if (err != null) {
          return res.status(500).end('Failed to save configuration: ' + err);
        } else {
          return res.redirect('/simulations/edit/' + encodeURIComponent(id));
        }
      });
    });
    return api = {
      init: function(cb) {
        var httpServer;
        cb = _.once(cb);
        httpServer = server.listen(app.config.controlServer.port);
        httpServer.once('listening', cb);
        return httpServer.once('error', cb);
      }
    };
  };

  module.exports = createControlServer;

}).call(this);
